@@ -495,16 +495,16 @@ uint64_t ssh_filesize(const char *path)
   return res;
 }
 
-rdirectory *ssh_read_directory(const char *path)
+rdirectory* ssh_read_directory(const char *path)
 {
-  char *p = ftp_path_absolute(path);
+  char* p = ftp_path_absolute(path);
   stripslash(p);
 
   sftp_dir dir = sftp_opendir(ftp->sftp_session, p);
   if (!dir)
   {
     free(p);
-    return 0;
+    return NULL;
   }
 
   ftp_trace("*** start parsing directory listing ***\n");
@@ -556,6 +556,7 @@ rdirectory *ssh_read_directory(const char *path)
 
   sftp_closedir(dir);
   rdir->path = p;
+  rdir_sort(rdir);
   ftp_trace("added directory '%s' to cache\n", p);
   list_additem(ftp->cache, rdir);
 
@@ -622,11 +623,11 @@ static int do_scp_read(ssh_scp scp, const char* infile, FILE* fp,
   int rc = ssh_scp_pull_request(scp);
   if (rc != SSH_SCP_REQUEST_NEWFILE)
   {
-    ftp_err(_("Failed to start scp download: %s\n"),
-            ssh_get_error(ftp->session));
+    ftp_trace("Failed to start scp download: %s\n",
+              ssh_get_error(ftp->session));
     ssh_scp_close(scp);
     ssh_scp_free(scp);
-    return -1;
+    return -2;
   }
 
   size_t size = ssh_scp_request_get_size(scp);
@@ -697,8 +698,13 @@ static int do_read(const char* infile, FILE* fp, transfer_mode_t mode,
     {
       int rc = ssh_scp_init(scp);
       if (rc == SSH_OK)
-        return do_scp_read(scp, infile, fp, mode, hookf);
-      ssh_scp_free(scp);
+      {
+        rc = do_scp_read(scp, infile, fp, mode, hookf);
+        if (rc != -2)
+          return rc;
+      }
+      else
+        ssh_scp_free(scp);
     }
   }
 
@@ -826,7 +832,7 @@ static int do_scp_write(ssh_scp scp, const char* path, FILE* fp,
             ssh_get_error(ftp->session));
     ssh_scp_close(scp);
     ssh_scp_free(scp);
-    return -1;
+    return -2;
   }
 
   /* read file */
@@ -885,8 +891,13 @@ static int do_write(const char* path, FILE* fp, ftp_transfer_func hookf,
     {
       int rc = ssh_scp_init(scp);
       if (rc == SSH_OK)
-        return do_scp_write(scp, path, fp, hookf);
-      ssh_scp_free(scp);
+      {
+        rc = do_scp_write(scp, path, fp, hookf);
+        if (rc != -2)
+          return rc;
+      }
+      else
+        ssh_scp_free(scp);
     }
   }
 
